@@ -210,7 +210,7 @@ DistributedTableSizeOnWorker(WorkerNode *workerNode, Oid relationId, char *sizeQ
 	char *tableSizeString;
 	uint64 tableSize = 0;
 	MultiConnection *connection = NULL;
-	uint32 connectionFlag = FORCE_NEW_CONNECTION;
+	uint32 connectionFlag = 0;
 	PGresult *result = NULL;
 	int queryResult = 0;
 	List *sizeList = NIL;
@@ -637,7 +637,7 @@ NodeGroupHasShardPlacements(uint32 groupId, bool onlyConsiderActivePlacements)
 
 	HeapTuple heapTuple = NULL;
 	SysScanDesc scanDescriptor = NULL;
-	ScanKeyData scanKey[scanKeyCount];
+	ScanKeyData scanKey[2];
 
 	Relation pgPlacement = heap_open(DistPlacementRelationId(),
 									 AccessShareLock);
@@ -1097,7 +1097,7 @@ DeleteShardPlacementRow(uint64 placementId)
 	Relation pgDistPlacement = NULL;
 	SysScanDesc scanDescriptor = NULL;
 	const int scanKeyCount = 1;
-	ScanKeyData scanKey[scanKeyCount];
+	ScanKeyData scanKey[1];
 	bool indexOK = true;
 	HeapTuple heapTuple = NULL;
 	TupleDesc tupleDescriptor = NULL;
