@@ -62,12 +62,15 @@
 #include "distributed/multi_logical_planner.h"
 #include "distributed/multi_router_planner.h"
 #include "distributed/multi_physical_planner.h"
-#include "distributed/recursive_planning.h"
 #include "distributed/multi_server_executor.h"
+#include "distributed/query_colocation_checker.h"
+#include "distributed/recursive_planning.h"
 #include "distributed/relation_restriction_equivalence.h"
 #include "lib/stringinfo.h"
 #include "optimizer/planner.h"
+#include "optimizer/prep.h"
 #include "parser/parsetree.h"
+#include "nodes/makefuncs.h"
 #include "nodes/nodeFuncs.h"
 #include "nodes/nodes.h"
 #include "nodes/pg_list.h"
@@ -86,6 +89,7 @@ typedef struct RecursivePlanningContext
 {
 	int level;
 	uint64 planId;
+	bool allDistributionKeysInQueryAreEqual; /* used for some optimizations */
 	List *subPlanList;
 	PlannerRestrictionContext *plannerRestrictionContext;
 } RecursivePlanningContext;
@@ -115,10 +119,35 @@ typedef struct VarLevelsUpWalkerContext
 static DeferredErrorMessage * RecursivelyPlanSubqueriesAndCTEs(Query *query,
 															   RecursivePlanningContext *
 															   context);
+static bool ShouldRecursivelyPlanNonColocatedSubqueries(Query *subquery,
+														RecursivePlanningContext *
+														context);
+static bool ContainsSubquery(Query *query);
+static void RecursivelyPlanNonColocatedSubqueries(Query *subquery,
+												  RecursivePlanningContext *context);
+static void RecursivelyPlanNonColocatedJoinWalker(Node *joinNode,
+												  ColocatedJoinChecker *
+												  colocatedJoinChecker,
+												  RecursivePlanningContext *
+												  recursivePlanningContext);
+static void RecursivelyPlanNonColocatedSubqueriesInWhere(Query *query,
+														 ColocatedJoinChecker *
+														 colocatedJoinChecker,
+														 RecursivePlanningContext *
+														 recursivePlanningContext);
+static List * SublinkList(Query *originalQuery);
+static bool ExtractSublinkWalker(Node *node, List **sublinkList);
+static bool ShouldRecursivelyPlanAllSubqueriesInWhere(Query *query);
+static bool RecursivelyPlanAllSubqueries(Node *node,
+										 RecursivePlanningContext *planningContext);
 static DeferredErrorMessage * RecursivelyPlanCTEs(Query *query,
 												  RecursivePlanningContext *context);
 static bool RecursivelyPlanSubqueryWalker(Node *node, RecursivePlanningContext *context);
-static bool ShouldRecursivelyPlanSubquery(Query *subquery);
+static bool ShouldRecursivelyPlanSubquery(Query *subquery,
+										  RecursivePlanningContext *context);
+static bool AllDistributionKeysInSubqueryAreEqual(Query *subquery,
+												  PlannerRestrictionContext *
+												  restrictionContext);
 static bool ShouldRecursivelyPlanSetOperation(Query *query,
 											  RecursivePlanningContext *context);
 static void RecursivelyPlanSetOperations(Query *query, Node *node,
@@ -159,6 +188,21 @@ GenerateSubplansForSubqueriesAndCTEs(uint64 planId, Query *originalQuery,
 	context.subPlanList = NIL;
 	context.plannerRestrictionContext = plannerRestrictionContext;
 
+	/*
+	 * Calculating the distribution key equality upfront is a trade-off for us.
+	 *
+	 * When the originalQuery contains the distribution key equality, we'd be
+	 * able to skip further checks for each lower level subqueries (i.e., if the
+	 * all query contains distribution key equality, each subquery also contains
+	 * distribution key equality.)
+	 *
+	 * When the originalQuery doesn't contain the distribution key equality,
+	 * calculating this wouldn't help us at all, we should individually check
+	 * each each subquery and subquery joins among subqueries.
+	 */
+	context.allDistributionKeysInQueryAreEqual =
+		AllDistributionKeysInQueryAreEqual(originalQuery, plannerRestrictionContext);
+
 	error = RecursivelyPlanSubqueriesAndCTEs(originalQuery, &context);
 	if (error != NULL)
 	{
@@ -238,10 +282,371 @@ RecursivelyPlanSubqueriesAndCTEs(Query *query, RecursivePlanningContext *context
 		RecursivelyPlanSetOperations(query, (Node *) query->setOperations, context);
 	}
 
+	/*
+	 * If the FROM clause is recurring (does not contain a distributed table),
+	 * then we cannot have any distributed tables appearing in subqueries in
+	 * the WHERE clause.
+	 */
+	if (ShouldRecursivelyPlanAllSubqueriesInWhere(query))
+	{
+		/* replace all subqueries in the WHERE clause */
+		RecursivelyPlanAllSubqueries((Node *) query->jointree->quals, context);
+	}
+
+	/*
+	 * If the query doesn't have distribution key equality,
+	 * recursively plan some of its subqueries.
+	 */
+	if (ShouldRecursivelyPlanNonColocatedSubqueries(query, context))
+	{
+		RecursivelyPlanNonColocatedSubqueries(query, context);
+	}
+
 	return NULL;
 }
 
 
+/*
+ * ShouldRecursivelyPlanNonColocatedSubqueries returns true if the input query contains joins
+ * that are not on the distribution key.
+ * *
+ * Note that at the point that this function is called, we've already recursively planned all
+ * the leaf subqueries. Thus, we're actually checking whether the joins among the subqueries
+ * on the distribution key or not.
+ */
+static bool
+ShouldRecursivelyPlanNonColocatedSubqueries(Query *subquery,
+											RecursivePlanningContext *context)
+{
+	/*
+	 * If the input query already contains the equality, simply return since it is not
+	 * possible to find any non colocated subqueries.
+	 */
+	if (context->allDistributionKeysInQueryAreEqual)
+	{
+		return false;
+	}
+
+	/*
+	 * This check helps us in two ways:
+	 *   (i) We're not targeting queries that don't include subqueries at all,
+	 *       they should go through regular planning.
+	 *  (ii) Lower level subqueries are already recursively planned, so we should
+	 *       only bother non-colocated subquery joins, which only happens when
+	 *       there are subqueries.
+	 */
+	if (!ContainsSubquery(subquery))
+	{
+		return false;
+	}
+
+	/* direct joins with local tables are not supported by any of Citus planners */
+	if (FindNodeCheckInRangeTableList(subquery->rtable, IsLocalTableRTE))
+	{
+		return false;
+	}
+
+	/*
+	 * Finally, check whether this subquery contains distribution key equality or not.
+	 */
+	if (!AllDistributionKeysInSubqueryAreEqual(subquery,
+											   context->plannerRestrictionContext))
+	{
+		return true;
+	}
+
+	return false;
+}
+
+
+/*
+ * ContainsSubquery returns true if the input query contains any subqueries
+ * in the FROM or WHERE clauses.
+ */
+static bool
+ContainsSubquery(Query *query)
+{
+	return JoinTreeContainsSubquery(query) || WhereClauseContainsSubquery(query);
+}
+
+
+/*
+ * RecursivelyPlanNonColocatedSubqueries gets a query which includes one or more
+ * other subqueries that are not joined on their distribution keys. The function
+ * tries to recursively plan some of the subqueries to make the input query
+ * executable by Citus.
+ *
+ * The function picks an anchor subquery and iterates on the remaining subqueries.
+ * Whenever it finds a non colocated subquery with the anchor subquery, the function
+ * decides to recursively plan the non colocated subquery.
+ *
+ * The function first handles subqueries in FROM clause (i.e., jointree->fromlist) and then
+ * subqueries in WHERE clause (i.e., jointree->quals).
+ *
+ * The function does not treat outer joins seperately. Thus, we might end up with
+ * a query where the function decides to recursively plan an outer side of an outer
+ * join (i.e., LEFT side of LEFT JOIN). For simplicity, we chose to do so and handle
+ * outer joins with a seperate pass on the join tree.
+ */
+static void
+RecursivelyPlanNonColocatedSubqueries(Query *subquery, RecursivePlanningContext *context)
+{
+	ColocatedJoinChecker colocatedJoinChecker;
+
+	FromExpr *joinTree = subquery->jointree;
+	PlannerRestrictionContext *restrictionContext = NULL;
+
+	/* create the context for the non colocated subquery planning */
+	restrictionContext = context->plannerRestrictionContext;
+	colocatedJoinChecker = CreateColocatedJoinChecker(subquery, restrictionContext);
+
+	/*
+	 * Although this is a rare case, we weren't able to pick an anchor
+	 * range table entry, so we cannot continue.
+	 */
+	if (colocatedJoinChecker.anchorRelationRestrictionList == NIL)
+	{
+		return;
+	}
+
+	/* handle from clause subqueries first */
+	RecursivelyPlanNonColocatedJoinWalker((Node *) joinTree, &colocatedJoinChecker,
+										  context);
+
+	/* handle subqueries in WHERE clause */
+	RecursivelyPlanNonColocatedSubqueriesInWhere(subquery, &colocatedJoinChecker,
+												 context);
+}
+
+
+/*
+ * RecursivelyPlanNonColocatedJoinWalker gets a join node and walks over it to find
+ * subqueries that live under the node.
+ *
+ * When a subquery found, its checked whether the subquery is colocated with the
+ * anchor subquery specified in the nonColocatedJoinContext. If not,
+ * the subquery is recursively planned.
+ */
+static void
+RecursivelyPlanNonColocatedJoinWalker(Node *joinNode,
+									  ColocatedJoinChecker *colocatedJoinChecker,
+									  RecursivePlanningContext *recursivePlanningContext)
+{
+	if (joinNode == NULL)
+	{
+		return;
+	}
+	else if (IsA(joinNode, FromExpr))
+	{
+		FromExpr *fromExpr = (FromExpr *) joinNode;
+		ListCell *fromExprCell;
+
+		/*
+		 * For each element of the from list, check whether the element is
+		 * colocated with the anchor subquery by recursing until we
+		 * find the subqueries.
+		 */
+		foreach(fromExprCell, fromExpr->fromlist)
+		{
+			Node *fromElement = (Node *) lfirst(fromExprCell);
+
+			RecursivelyPlanNonColocatedJoinWalker(fromElement, colocatedJoinChecker,
+												  recursivePlanningContext);
+		}
+	}
+	else if (IsA(joinNode, JoinExpr))
+	{
+		JoinExpr *joinExpr = (JoinExpr *) joinNode;
+
+		/* recurse into the left subtree */
+		RecursivelyPlanNonColocatedJoinWalker(joinExpr->larg, colocatedJoinChecker,
+											  recursivePlanningContext);
+
+		/* recurse into the right subtree */
+		RecursivelyPlanNonColocatedJoinWalker(joinExpr->rarg, colocatedJoinChecker,
+											  recursivePlanningContext);
+	}
+	else if (IsA(joinNode, RangeTblRef))
+	{
+		int rangeTableIndex = ((RangeTblRef *) joinNode)->rtindex;
+		List *rangeTableList = colocatedJoinChecker->subquery->rtable;
+		RangeTblEntry *rte = rt_fetch(rangeTableIndex, rangeTableList);
+		Query *subquery = NULL;
+
+		/* we're only interested in subqueries for now */
+		if (rte->rtekind != RTE_SUBQUERY)
+		{
+			return;
+		}
+
+		/*
+		 * If the subquery is not colocated with the anchor subquery,
+		 * recursively plan it.
+		 */
+		subquery = rte->subquery;
+		if (!SubqueryColocated(subquery, colocatedJoinChecker))
+		{
+			RecursivelyPlanSubquery(subquery, recursivePlanningContext);
+		}
+	}
+	else
+	{
+		pg_unreachable();
+	}
+}
+
+
+/*
+ * RecursivelyPlanNonColocatedJoinWalker gets a query and walks over its sublinks
+ * to find subqueries that live in WHERE clause.
+ *
+ * When a subquery found, its checked whether the subquery is colocated with the
+ * anchor subquery specified in the nonColocatedJoinContext. If not,
+ * the subquery is recursively planned.
+ */
+static void
+RecursivelyPlanNonColocatedSubqueriesInWhere(Query *query,
+											 ColocatedJoinChecker *colocatedJoinChecker,
+											 RecursivePlanningContext *
+											 recursivePlanningContext)
+{
+	List *sublinkList = SublinkList(query);
+	ListCell *sublinkCell = NULL;
+
+	foreach(sublinkCell, sublinkList)
+	{
+		SubLink *sublink = (SubLink *) lfirst(sublinkCell);
+		Query *subselect = (Query *) sublink->subselect;
+
+		/* subselect is probably never NULL, but anyway lets keep the check */
+		if (subselect == NULL)
+		{
+			continue;
+		}
+
+		if (!SubqueryColocated(subselect, colocatedJoinChecker))
+		{
+			RecursivelyPlanSubquery(subselect, recursivePlanningContext);
+		}
+	}
+}
+
+
+/*
+ * SublinkList finds the subquery nodes in the where clause of the given query. Note
+ * that the function should be called on the original query given that postgres
+ * standard_planner() may convert the subqueries in WHERE clause to joins.
+ */
+static List *
+SublinkList(Query *originalQuery)
+{
+	FromExpr *joinTree = originalQuery->jointree;
+	Node *queryQuals = NULL;
+	List *sublinkList = NIL;
+
+	if (!joinTree)
+	{
+		return NIL;
+	}
+
+	queryQuals = joinTree->quals;
+	ExtractSublinkWalker(queryQuals, &sublinkList);
+
+	return sublinkList;
+}
+
+
+/*
+ * ExtractSublinkWalker walks over a quals node, and finds all sublinks
+ * in that node.
+ */
+static bool
+ExtractSublinkWalker(Node *node, List **sublinkList)
+{
+	bool walkerResult = false;
+	if (node == NULL)
+	{
+		return false;
+	}
+
+	if (IsA(node, SubLink))
+	{
+		(*sublinkList) = lappend(*sublinkList, node);
+	}
+	else
+	{
+		walkerResult = expression_tree_walker(node, ExtractSublinkWalker,
+											  sublinkList);
+	}
+
+	return walkerResult;
+}
+
+
+/*
+ * ShouldRecursivelyPlanAllSubqueriesInWhere returns true if the query has
+ * a WHERE clause and a recurring FROM clause (does not contain a distributed
+ * table).
+ */
+static bool
+ShouldRecursivelyPlanAllSubqueriesInWhere(Query *query)
+{
+	FromExpr *joinTree = NULL;
+	Node *whereClause = NULL;
+
+	joinTree = query->jointree;
+	if (joinTree == NULL)
+	{
+		/* there is no FROM clause */
+		return false;
+	}
+
+	whereClause = joinTree->quals;
+	if (whereClause == NULL)
+	{
+		/* there is no WHERE clause */
+		return false;
+	}
+
+	if (FindNodeCheckInRangeTableList(query->rtable, IsDistributedTableRTE))
+	{
+		/* there is a distributed table in the FROM clause */
+		return false;
+	}
+
+	return true;
+}
+
+
+/*
+ * RecursivelyPlanAllSubqueries descends into an expression tree and recursively
+ * plans all subqueries that contain at least one distributed table. The recursive
+ * planning starts from the top of the input query.
+ */
+static bool
+RecursivelyPlanAllSubqueries(Node *node, RecursivePlanningContext *planningContext)
+{
+	if (node == NULL)
+	{
+		return false;
+	}
+
+	if (IsA(node, Query))
+	{
+		Query *query = (Query *) node;
+
+		if (FindNodeCheckInRangeTableList(query->rtable, IsDistributedTableRTE))
+		{
+			RecursivelyPlanSubquery(query, planningContext);
+		}
+
+		return false;
+	}
+
+	return expression_tree_walker(node, RecursivelyPlanAllSubqueries, planningContext);
+}
+
+
 /*
  * RecursivelyPlanCTEs plans all CTEs in the query by recursively calling the planner
  * The resulting plan is added to planningContext->subPlanList and CTE references
@@ -262,15 +667,6 @@ RecursivelyPlanCTEs(Query *query, RecursivePlanningContext *planningContext)
 		return NULL;
 	}
 
-	if (query->hasModifyingCTE)
-	{
-		/* we could easily support these, but it's a little scary */
-		return DeferredError(ERRCODE_FEATURE_NOT_SUPPORTED,
-							 "data-modifying statements are not supported in "
-							 "the WITH clauses of distributed queries",
-							 NULL, NULL);
-	}
-
 	if (query->hasRecursive)
 	{
 		return DeferredError(ERRCODE_FEATURE_NOT_SUPPORTED,
@@ -419,7 +815,7 @@ RecursivelyPlanSubqueryWalker(Node *node, RecursivePlanningContext *context)
 		 * Recursively plan this subquery if it cannot be pushed down and is
 		 * eligible for recursive planning.
 		 */
-		if (ShouldRecursivelyPlanSubquery(query))
+		if (ShouldRecursivelyPlanSubquery(query, context))
 		{
 			RecursivelyPlanSubquery(query, context);
 		}
@@ -439,7 +835,7 @@ RecursivelyPlanSubqueryWalker(Node *node, RecursivePlanningContext *context)
  * For the details, see the cases in the function.
  */
 static bool
-ShouldRecursivelyPlanSubquery(Query *subquery)
+ShouldRecursivelyPlanSubquery(Query *subquery, RecursivePlanningContext *context)
 {
 	if (FindNodeCheckInRangeTableList(subquery->rtable, IsLocalTableRTE))
 	{
@@ -455,6 +851,23 @@ ShouldRecursivelyPlanSubquery(Query *subquery)
 	}
 	else if (DeferErrorIfCannotPushdownSubquery(subquery, false) == NULL)
 	{
+		/*
+		 * We should do one more check for the distribution key equality.
+		 *
+		 * If the input query to the planner doesn't contain distribution key equality,
+		 * we should further check whether this individual subquery contains or not.
+		 *
+		 * If all relations are not joined on their distribution keys for the given
+		 * subquery, we cannot push push it down and therefore we should try to
+		 * recursively plan it.
+		 */
+		if (!context->allDistributionKeysInQueryAreEqual &&
+			!AllDistributionKeysInSubqueryAreEqual(subquery,
+												   context->plannerRestrictionContext))
+		{
+			return true;
+		}
+
 		/*
 		 * Citus can pushdown this subquery, no need to recursively
 		 * plan which is much expensive than pushdown.
@@ -475,6 +888,39 @@ ShouldRecursivelyPlanSubquery(Query *subquery)
 }
 
 
+/*
+ * AllDistributionKeysInSubqueryAreEqual is a wrapper function
+ * for AllDistributionKeysInQueryAreEqual(). Here, we filter the
+ * planner restrictions for the given subquery and do the restriction
+ * equality checks on the filtered restriction.
+ */
+static bool
+AllDistributionKeysInSubqueryAreEqual(Query *subquery,
+									  PlannerRestrictionContext *restrictionContext)
+{
+	bool allDistributionKeysInSubqueryAreEqual = false;
+	PlannerRestrictionContext *filteredRestrictionContext = NULL;
+
+	/* we don't support distribution eq. checks for CTEs yet */
+	if (subquery->cteList != NIL)
+	{
+		return false;
+	}
+
+	filteredRestrictionContext =
+		FilterPlannerRestrictionForQuery(restrictionContext, subquery);
+
+	allDistributionKeysInSubqueryAreEqual =
+		AllDistributionKeysInQueryAreEqual(subquery, filteredRestrictionContext);
+	if (!allDistributionKeysInSubqueryAreEqual)
+	{
+		return false;
+	}
+
+	return true;
+}
+
+
 /*
  * ShouldRecursivelyPlanSetOperation determines whether the leaf queries of a
  * set operations tree need to be recursively planned in order to support the
@@ -880,8 +1326,18 @@ BuildSubPlanResultQuery(Query *subquery, List *columnAliasList, uint64 planId,
 	Oid copyFormatId = BinaryCopyFormatId();
 	int columnAliasCount = list_length(columnAliasList);
 
+	List *targetEntryList = NIL;
+	if (subquery->returningList)
+	{
+		targetEntryList = subquery->returningList;
+	}
+	else
+	{
+		targetEntryList = subquery->targetList;
+	}
+
 	/* build the target list and column definition list */
-	foreach(targetEntryCell, subquery->targetList)
+	foreach(targetEntryCell, targetEntryList)
 	{
 		TargetEntry *targetEntry = (TargetEntry *) lfirst(targetEntryCell);
 		Node *targetExpr = (Node *) targetEntry->expr;
