@@ -62,6 +62,7 @@ StartRemoteTransactionBegin(struct MultiConnection *connection)
 	DistributedTransactionId *distributedTransactionId = NULL;
 	ListCell *subIdCell = NULL;
 	List *activeSubXacts = NIL;
+	const char *timestamp = NULL;
 
 	Assert(transaction->transactionState == REMOTE_TRANS_INVALID);
 
@@ -84,12 +85,13 @@ StartRemoteTransactionBegin(struct MultiConnection *connection)
 	 * seperate roundtrips for these two statements.
 	 */
 	distributedTransactionId = GetCurrentDistributedTransactionId();
+	timestamp = timestamptz_to_str(distributedTransactionId->timestamp);
 	appendStringInfo(beginAndSetDistributedTransactionId,
 					 "SELECT assign_distributed_transaction_id(%d, " UINT64_FORMAT
 					 ", '%s');",
 					 distributedTransactionId->initiatorNodeIdentifier,
 					 distributedTransactionId->transactionNumber,
-					 timestamptz_to_str(distributedTransactionId->timestamp));
+					 timestamp);
 
 	/* append in-progress savepoints for this transaction */
 	activeSubXacts = ActiveSubXacts();
@@ -380,7 +382,14 @@ StartRemoteTransactionAbort(MultiConnection *connection)
 	}
 	else
 	{
-		if (!NonblockingForgetResults(connection))
+		/*
+		 * In case of a cancellation, the connection might still be working
+		 * on some commands. Try to consume the results such that the
+		 * connection can be reused, but do not want to wait for commands
+		 * to finish. Instead we just close the connection if the command
+		 * is still busy.
+		 */
+		if (!ClearResultsIfReady(connection))
 		{
 			ShutdownConnection(connection);
 
