@@ -25,7 +25,7 @@
 #include "distributed/errormessage.h"
 #include "distributed/multi_logical_planner.h"
 #include "distributed/multi_physical_planner.h"
-#include "distributed/multi_planner.h"
+#include "distributed/distributed_planner.h"
 #include "distributed/multi_server_executor.h"
 #include "distributed/master_metadata_utility.h"
 #include "lib/stringinfo.h"
@@ -70,10 +70,6 @@
 #define WRITE_OID_FIELD(fldname) \
 	appendStringInfo(str, " :" CppAsString(fldname) " %u", node->fldname)
 
-/* Write a long-integer field */
-#define WRITE_LONG_FIELD(fldname) \
-	appendStringInfo(str, " :" CppAsString(fldname) " %ld", node->fldname)
-
 /* Write a char field (ie, one ascii character) */
 #define WRITE_CHAR_FIELD(fldname) \
 	appendStringInfo(str, " :" CppAsString(fldname) " %c", node->fldname)
@@ -173,12 +169,13 @@ OutMultiTreeRoot(OUTFUNC_ARGS)
 
 
 void
-OutMultiPlan(OUTFUNC_ARGS)
+OutDistributedPlan(OUTFUNC_ARGS)
 {
-	WRITE_LOCALS(MultiPlan);
+	WRITE_LOCALS(DistributedPlan);
 
-	WRITE_NODE_TYPE("MULTIPLAN");
+	WRITE_NODE_TYPE("DISTRIBUTEDPLAN");
 
+	WRITE_UINT64_FIELD(planId);
 	WRITE_INT_FIELD(operation);
 	WRITE_BOOL_FIELD(hasReturning);
 
@@ -191,10 +188,24 @@ OutMultiPlan(OUTFUNC_ARGS)
 	WRITE_NODE_FIELD(insertTargetList);
 	WRITE_OID_FIELD(targetRelationId);
 
+	WRITE_NODE_FIELD(subPlanList);
+
 	WRITE_NODE_FIELD(planningError);
 }
 
 
+void
+OutDistributedSubPlan(OUTFUNC_ARGS)
+{
+	WRITE_LOCALS(DistributedSubPlan);
+
+	WRITE_NODE_TYPE("DISTRIBUTEDSUBPLAN");
+
+	WRITE_UINT_FIELD(subPlanId);
+	WRITE_NODE_FIELD(plan);
+}
+
+
 void
 OutMultiProject(OUTFUNC_ARGS)
 {
