@@ -65,7 +65,7 @@ static bool ReceiveRegularFile(const char *nodeName, uint32 nodePort,
 							   StringInfo filePath);
 static void ReceiveResourceCleanup(int32 connectionId, const char *filename,
 								   int32 fileDescriptor);
-static void DeleteFile(const char *filename);
+static void CitusDeleteFile(const char *filename);
 static void FetchTableCommon(text *tableName, uint64 remoteTableSize,
 							 ArrayType *nodeNameObject, ArrayType *nodePortObject,
 							 bool (*FetchTableFunction)(const char *, uint32,
@@ -343,7 +343,7 @@ ReceiveRegularFile(const char *nodeName, uint32 nodePort, const char *nodeUser,
 	/* loop until we receive and append all the data from remote node */
 	while (!copyDone)
 	{
-		CopyStatus copyStatus = MultiClientCopyData(connectionId, fileDescriptor);
+		CopyStatus copyStatus = MultiClientCopyData(connectionId, fileDescriptor, NULL);
 		if (copyStatus == CLIENT_COPY_DONE)
 		{
 			copyDone = true;
@@ -370,7 +370,7 @@ ReceiveRegularFile(const char *nodeName, uint32 nodePort, const char *nodeUser,
 						  errmsg("could not close file \"%s\": %m", filename)));
 
 		/* if we failed to close file, try to delete it before erroring out */
-		DeleteFile(filename);
+		CitusDeleteFile(filename);
 
 		return false;
 	}
@@ -418,7 +418,7 @@ ReceiveResourceCleanup(int32 connectionId, const char *filename, int32 fileDescr
 
 /* Deletes file with the given filename. */
 static void
-DeleteFile(const char *filename)
+CitusDeleteFile(const char *filename)
 {
 	int deleted = unlink(filename);
 	if (deleted != 0)
@@ -784,19 +784,14 @@ ExtractShardId(const char *tableName)
 	}
 	shardIdString++;
 
-#ifdef HAVE_STRTOULL
 	errno = 0;
-	shardId = strtoull(shardIdString, &shardIdStringEnd, 0);
+	shardId = pg_strtouint64(shardIdString, &shardIdStringEnd, 0);
 
 	if (errno != 0 || (*shardIdStringEnd != '\0'))
 	{
 		ereport(ERROR, (errmsg("could not extract shardId from table name \"%s\"",
 							   tableName)));
 	}
-#else
-	ereport(ERROR, (errmsg("could not extract shardId from table name"),
-					errhint("Your platform does not support strtoull()")));
-#endif
 
 	return shardId;
 }
@@ -894,7 +889,7 @@ FetchRegularTable(const char *nodeName, uint32 nodePort, const char *tableName)
 						PROCESS_UTILITY_TOPLEVEL, NULL, None_Receiver, NULL);
 
 	/* finally delete the temporary file we created */
-	DeleteFile(localFilePath->data);
+	CitusDeleteFile(localFilePath->data);
 
 	SetUserIdAndSecContext(savedUserId, savedSecurityContext);
 
@@ -1289,7 +1284,7 @@ worker_append_table_to_shard(PG_FUNCTION_ARGS)
 						PROCESS_UTILITY_TOPLEVEL, NULL, None_Receiver, NULL);
 
 	/* finally delete the temporary file we created */
-	DeleteFile(localFilePath->data);
+	CitusDeleteFile(localFilePath->data);
 
 	PG_RETURN_VOID();
 }
@@ -1380,10 +1375,10 @@ AlterSequenceMinMax(Oid sequenceId, char *schemaName, char *sequenceName)
 		 * DefElem->arg can only hold literal ints up to int4, in order to represent
 		 * larger numbers we need to construct a float represented as a string.
 		 */
-		appendStringInfo(startNumericString, "%lu", startValue);
+		appendStringInfo(startNumericString, INT64_FORMAT, startValue);
 		startFloatArg = (Node *) makeFloat(startNumericString->data);
 
-		appendStringInfo(maxNumericString, "%lu", maxValue);
+		appendStringInfo(maxNumericString, INT64_FORMAT, maxValue);
 		maxFloatArg = (Node *) makeFloat(maxNumericString->data);
 
 		SetDefElemArg(alterSequenceStatement, "start", startFloatArg);
