@@ -176,6 +176,8 @@ typedef struct MultiExtendedOp
 	Node *havingQual;
 	List *distinctClause;
 	bool hasDistinctOn;
+	bool hasWindowFuncs;
+	List *windowClause;
 } MultiExtendedOp;
 
 
@@ -187,17 +189,18 @@ extern bool SubqueryPushdown;
 extern MultiTreeRoot * MultiLogicalPlanCreate(Query *originalQuery, Query *queryTree,
 											  PlannerRestrictionContext *
 											  plannerRestrictionContext);
+extern bool JoinTreeContainsSubquery(Query *query);
+extern bool WhereClauseContainsSubquery(Query *query);
+extern bool FindNodeCheck(Node *node, bool (*check)(Node *));
 extern bool SingleRelationRepartitionSubquery(Query *queryTree);
 extern DeferredErrorMessage * DeferErrorIfCannotPushdownSubquery(Query *subqueryTree,
 																 bool
 																 outerMostQueryHasLimit);
 extern DeferredErrorMessage * DeferErrorIfUnsupportedUnionQuery(Query *queryTree);
-extern PlannerRestrictionContext * FilterPlannerRestrictionForQuery(
-	PlannerRestrictionContext *plannerRestrictionContext,
-	Query *query);
 extern bool SafeToPushdownWindowFunction(Query *query, StringInfo *errorDetail);
 extern bool TargetListOnPartitionColumn(Query *query, List *targetEntryList);
 extern bool FindNodeCheckInRangeTableList(List *rtable, bool (*check)(Node *));
+extern bool IsDistributedTableRTE(Node *node);
 extern bool QueryContainsDistributedTableRTE(Query *query);
 extern bool ContainsReadIntermediateResultFunction(Node *node);
 extern MultiNode * ParentNode(MultiNode *multiNode);
@@ -223,6 +226,7 @@ extern bool ExtractRangeTableRelationWalker(Node *node, List **rangeTableList);
 extern bool ExtractRangeTableEntryWalker(Node *node, List **rangeTableList);
 extern List * pull_var_clause_default(Node *node);
 extern bool OperatorImplementsEquality(Oid opno);
+extern bool FindNodeCheck(Node *node, bool (*check)(Node *));
 
 
 #endif   /* MULTI_LOGICAL_PLANNER_H */
