@@ -6,7 +6,7 @@
 -- and that we don't partition or join prune shards that have null values.
 
 
-ALTER SEQUENCE pg_catalog.pg_dist_shardid_seq RESTART 760000;
+SET citus.next_shard_id TO 760000;
 
 -- print whether we're using version > 9 to make version-specific tests clear
 SHOW server_version \gset
